"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var chalk_1 = require("chalk");
var deepmerge = require("deepmerge");
var fs = require("fs");
var os = require("os");
var path = require("path");
var internal_1 = require("../internal");
var configPath;
/** Resolves the timpla config. */
exports.resolveConfig = function () {
    // memoize to only call it once!
    if (configPath) {
        return configPath;
    }
    var defaultConfig = path.resolve(internal_1.TIMPLA_PROCESS.TIMPLA_DIR, '.timplaconfig.js');
    var userConfig = internal_1.projectPath('.timplaconfig.js');
    if (internal_1.TIMPLA_PROCESS.TIMPLA_CONFIG_FILE) {
        configPath = internal_1.projectPath(internal_1.TIMPLA_PROCESS.TIMPLA_CONFIG_FILE);
    }
    else if (fs.existsSync(userConfig)) {
        configPath = userConfig;
    }
    else {
        configPath = defaultConfig;
    }
    return configPath;
};
/**
 * The timpla config.
 */
exports.TIMPLA_CONFIG = internal_1.sureLazyImport(exports.resolveConfig());
exports.configure = function (timplaConfig) {
    var conf = internal_1.TIMPLA_DEFAULTS;
    if (timplaConfig) {
        conf = deepmerge(internal_1.TIMPLA_DEFAULTS, timplaConfig, { arrayMerge: internal_1.arrayMerge });
    }
    internal_1.logTitle(chalk_1.default.green("Timpla config used:"), chalk_1.default.blue(exports.resolveConfig()), chalk_1.default.yellow("Set the TIMPLA_CONFIG_PATH environment to point to a different config."));
    internal_1.devLoggerTitle("Config dump");
    internal_1.devLogger('%O', conf, os.EOL, os.EOL);
    return conf;
};
//# sourceMappingURL=TIMPLA_CONFIG.js.map