"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var chalk_1 = require("chalk");
var path = require("path");
var internal_1 = require("../internal");
var PROCESS_CWD = process.cwd();
var INIT_CWD = process.env.INIT_CWD;
/**
 * Symlinks and gulp sometimes fight with each other.
 * We can resolve to the true base dir of Timpla using this file!
 */
var TIMPLA_DIR = path.resolve(__dirname, '../../');
exports.TIMPLA_PROCESS = {
    BABEL_ENV: process.env.BABEL_ENV,
    INIT_CWD: INIT_CWD,
    IS_DEBUGGING: process.env.DEBUG === 'timpla',
    IS_LOCAL_TIMPLADEV: PROCESS_CWD === INIT_CWD,
    MEASURE: !!process.env.MEASURE,
    NODE_ENV: process.env.NODE_ENV || 'development',
    PROCESS_CWD: PROCESS_CWD,
    TIMPLA_CONFIG_FILE: process.env.TIMPLA_CONFIG_FILE,
    TIMPLA_DEV_RELOAD: process.env.TIMPLA_DEV_RELOAD,
    TIMPLA_DIR: TIMPLA_DIR,
    isDevelopment: process.env.NODE_ENV !== 'production',
    isProduction: process.env.NODE_ENV === 'production',
    isTimplaReloaded: process.env.TIMPLA_DEV_RELOAD === 'true',
};
/**
 * A function that can be used to resolve a path relatively to the
 * project directory.
 *
 * We often want to resolve paths relatively to the project root
 * directory. To do that, we use the `INIT_CWD` environment variable
 * provided by Gulp. This variable always resolves to the project
 * root directory so we use it as the seed path and then add the
 * remaining arguments passed and resolving everything using the
 * `path.resolve` function.
 *
 * The returned path is a fully resolved absolute path relative to
 * the project root directory.
 */
exports.projectPath = function () {
    var paths = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        paths[_i] = arguments[_i];
    }
    if (paths) {
        var cleaned = paths.filter(Boolean);
        return path.resolve.apply(path, __spread([exports.TIMPLA_PROCESS.INIT_CWD], cleaned));
    }
    return exports.TIMPLA_PROCESS.INIT_CWD;
};
var packageJsonFile = internal_1.sureLazyImport(path.resolve(exports.TIMPLA_PROCESS.INIT_CWD, 'package.json'));
var allDeps = __assign({}, packageJsonFile.dependencies, packageJsonFile.devDependencies);
var packagesInstalled = __spread(new Set(Object.keys(allDeps)));
/**
 * Checks if a package is installed.
 * Throws an error if it doesn't (forces the cli to exit).
 * Notifies the user, so that they can install missing deps!
 */
exports.isPackageInstalled = function (packageName) {
    if (!packagesInstalled.includes(packageName)) {
        throw Error(chalk_1.default.red("Whoops, please make sure you install " + packageName + ":\nnpm i --save-dev " + packageName));
    }
    return true;
};
/** Checks if a package is installed, prompts if a package is missing */
exports.lazyImport = function (packageName) {
    return exports.isPackageInstalled(packageName) && internal_1.sureLazyImport(packageName);
};
//# sourceMappingURL=timplaProcess.js.map