"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var chalk_1 = require("chalk");
var os = require("os");
var path = require("path");
var querystring = require("querystring");
var webpack = require("webpack");
var wbmerge = require("webpack-merge");
var internal_1 = require("../internal");
var invalidJsEntryMessage = "\nInvalid javascripts entry supplied. Timpla only accepts the object-style format:\n{\n  [name: string]: string[]\n}\n";
var ensureLeadingDot = function (x) {
    return x.indexOf('.') === 0 ? x : "." + x;
};
exports.webpackMultiConfig = function (timplaConfig) { return function (env) {
    var jsConfig = timplaConfig.javascripts;
    var jsSrc = internal_1.projectSrcPath(jsConfig.src);
    var jsDest = internal_1.projectDestPath(jsConfig.dest);
    var publicPath = internal_1.pathToUrl(jsConfig.publicPath || jsConfig.dest, '/');
    var useDevESLint = internal_1.TIMPLA_PROCESS.isDevelopment && !!jsConfig.development.eslint;
    var babelLoaderOptions = jsConfig.babelLoaderOptions;
    var esLintLoaderOptions = useDevESLint && jsConfig.development.eslint.esLintLoaderOptions;
    var useDevTSLint = internal_1.TIMPLA_PROCESS.isDevelopment && !!jsConfig.development.tslint;
    var forkTsCheckerOptions = useDevTSLint && jsConfig.development.tslint && jsConfig.development.tslint.forkTsCheckerOptions;
    var mode = internal_1.TIMPLA_PROCESS.BABEL_ENV || internal_1.TIMPLA_PROCESS.NODE_ENV || env;
    var useProdRev = internal_1.TIMPLA_PROCESS.isProduction && timplaConfig.production.rev;
    var webpackHotMiddlewareOptions = jsConfig.development.webpackHotMiddlewareOptions;
    var hotEnabled = !!webpackHotMiddlewareOptions;
    var useDevHmr = internal_1.TIMPLA_PROCESS.isDevelopment && hotEnabled;
    var webpackBundleAnalyzerOptions = jsConfig.webpackBundleAnalyzerOptions;
    var extensions = jsConfig.extensions.map(ensureLeadingDot);
    var resolve = function () {
        var filePaths = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            filePaths[_i] = arguments[_i];
        }
        var files = filePaths.filter(Boolean);
        if (!files.every(function (e) { return typeof e === 'string'; }) || !files.length) {
            throw Error(chalk_1.default.red('Invalid arguments passed to the Timpla resolve function. Please pass strings.'));
        }
        return filePaths.map(function (filePath) { return internal_1.projectSrcPath(jsConfig.src, filePath); });
    };
    /** Prepares the Webpack entry config */
    var getEntryObject = function (ent) {
        var derivedEntry = ent;
        if (typeof ent === 'function') {
            derivedEntry = ent({ resolve: resolve });
        }
        if (typeof derivedEntry !== 'object') {
            throw Error(chalk_1.default.red(invalidJsEntryMessage));
        }
        var normalisedArrays = Object.entries(derivedEntry).reduce(function (acc, _a) {
            var _b = __read(_a, 2), entryKey = _b[0], entryValue = _b[1];
            acc[entryKey] = Array.isArray(entryValue) ? entryValue : [entryValue];
            return acc;
        }, {});
        return normalisedArrays;
    };
    var webpackEntry = getEntryObject(jsConfig.entry);
    /**
     * Bundle analyzer can be disabled through a boolean.
     * If an object is provided, we use that and set defaults, so timpla can work accordingly.
     */
    var bundleAnalyzer = function () {
        if (!webpackBundleAnalyzerOptions) {
            return false;
        }
        var BundleAnalyzerPlugin = internal_1.lazyImport('webpack-bundle-analyzer').BundleAnalyzerPlugin;
        var bundleAnalyzerConfig = internal_1.getFirstValidObject(webpackBundleAnalyzerOptions, {});
        var merged = __assign({}, bundleAnalyzerConfig, (internal_1.TIMPLA_PROCESS.isDevelopment
            ? {
                analyzerMode: 'server',
                // To not drive the user insane, we have to check if we've reloaded timpla
                openAnalyzer: !internal_1.TIMPLA_PROCESS.isTimplaReloaded && bundleAnalyzerConfig.openAnalyzer,
            }
            : {
                // opening analyzerMode in prod causes the build process to hang
                // the user will need to manually execute npx timpla openAnalyzer to view the report
                analyzerMode: 'disabled',
                generateStatsFile: true,
            }));
        return new BundleAnalyzerPlugin(merged);
    };
    var wbConfigDefault = function () {
        return {
            context: internal_1.TIMPLA_PROCESS.INIT_CWD,
            entry: webpackEntry,
            // Tell Webpack to provide empty mocks for them so importing them works. // Some libraries import Node modules but don't use them in the browser.
            node: { dgram: 'empty', fs: 'empty', net: 'empty', tls: 'empty', child_process: 'empty' },
            optimization: { minimizer: [] },
            output: { path: path.normalize(jsDest), filename: '[name].js', publicPath: publicPath },
            // Turn off performance processing because we utilize
            // our own hints via the FileSizeReporter
            performance: false,
            plugins: [
                // Moment.js is an extremely popular library that bundles large locale files
                // by default due to how Webpack interprets its code. This is a practical
                // solution that requires the user to opt into importing specific locales.
                // https://github.com/jmblog/how-to-optimize-momentjs-with-webpack
                // You can remove this if you don't use Moment.js:
                new webpack.IgnorePlugin(/^\.\/locale$/, /moment$/),
                bundleAnalyzer(),
            ].filter(Boolean),
            resolve: { extensions: extensions, alias: {}, modules: [jsSrc, internal_1.projectPath('node_modules')] },
            resolveLoader: { modules: [internal_1.projectPath('node_modules')] },
        };
    };
    var wbConfigProd = function () {
        var HardSourceWebpackPlugin = internal_1.lazyImport('hard-source-webpack-plugin');
        var TerserPlugin = internal_1.sureLazyImport('terser-webpack-plugin');
        var prodConfig = jsConfig.production;
        var terserConfig = __assign({}, jsConfig.production.terserConfig, { sourceMap: !!prodConfig.devtool });
        var hardSourceOptions = internal_1.getFirstValidObject(prodConfig.hardSourceOptions, {});
        return {
            // Fail out on the first error instead of tolerating it.
            bail: true,
            devtool: prodConfig.devtool || false,
            module: { strictExportPresence: true },
            optimization: {
                minimizer: [new TerserPlugin(terserConfig)],
            },
            plugins: [hardSourceOptions && new HardSourceWebpackPlugin(hardSourceOptions)].filter(Boolean),
        };
    };
    var wbConfigRev = function () { return ({
        output: {
            filename: '[name]-[hash].js',
        },
        plugins: [internal_1.webpackManifest(jsConfig.dest, timplaConfig.dest)],
    }); };
    /**
     * Outputs an entry config that will be merged with the jsconfig above...
     */
    var wbConfigHmr = function () {
        var hotMiddleware = "webpack-hot-middleware/client?" + querystring.stringify(webpackHotMiddlewareOptions);
        var injectHmr = function (acc, key) {
            // Since we're merging with another array
            acc[key] = [hotMiddleware];
            return acc;
        };
        var injectedEntries = Object.keys(webpackEntry).reduce(injectHmr, {});
        return {
            entry: injectedEntries,
            plugins: [new webpack.HotModuleReplacementPlugin()],
        };
    };
    var wbConfigDev = function () { return ({
        devtool: jsConfig.development.devtool,
        output: {
            pathinfo: true,
        },
    }); };
    /**
     * There will be many ways to configure the babel-loader.
     * To make it fool-proof:
     * a) The entire webpack configuration will be available to configure in .timplaconfig.js
     * b) Pick up the user's babelrc file from their project root
     */
    var wbConfigBabelLoader = function () {
        return {
            module: {
                rules: [
                    {
                        exclude: /node_modules/,
                        test: new RegExp("(\\" + extensions.join('$|') + "$)"),
                        use: {
                            loader: 'babel-loader',
                            options: __assign({ 
                                // Don't waste time on Gzipping the cache
                                cacheCompression: false, 
                                // Enable caching results in ./node_modules/.cache/babel-loader/
                                // directory for faster rebuilds.
                                cacheDirectory: true, 
                                // Set the root to where the project src
                                root: internal_1.TIMPLA_PROCESS.INIT_CWD }, babelLoaderOptions),
                        },
                    },
                ],
            },
        };
    };
    var wbConfigESLint = function () {
        internal_1.isPackageInstalled('eslint-loader');
        return {
            module: {
                rules: [
                    {
                        exclude: /node_modules/,
                        test: new RegExp("(\\" + extensions.filter(function (ext) { return !ext.includes('ts'); }).join('$|') + "$)"),
                        use: {
                            loader: 'eslint-loader',
                            options: __assign({ cache: true, formatter: internal_1.esWbformatter({
                                    basePath: timplaConfig.src,
                                }) }, esLintLoaderOptions),
                        },
                    },
                ],
            },
        };
    };
    /**
     * In order to enable TypeScript, add the @babel/preset-typescript
     * TypeScript and TSLint are set as peerDependencies,
     * so you can bring over your preferred versions
     */
    var wbConfigTSLint = function () {
        var ForkTsCheckerWebpackPlugin = internal_1.lazyImport('fork-ts-checker-webpack-plugin');
        return {
            plugins: [
                new ForkTsCheckerWebpackPlugin(__assign({ async: false, formatter: internal_1.tsWbformatter, tslint: true, watch: jsSrc }, forkTsCheckerOptions)),
            ],
        };
    };
    var logHmrMessage = function (isHot) {
        var hmrColor = isHot ? 'green' : 'yellow';
        var hmrMessage = isHot ? 'running' : 'turned off';
        internal_1.logTitle(chalk_1.default[hmrColor]("Webpack: Hot Module Reload " + hmrMessage + "."));
    };
    // All configs are functions
    var configs = [
        wbConfigDefault,
        wbConfigBabelLoader,
        useDevESLint && wbConfigESLint,
        useDevTSLint && wbConfigTSLint,
        useDevHmr && wbConfigHmr,
        internal_1.TIMPLA_PROCESS.isProduction && wbConfigProd,
        useProdRev && wbConfigRev,
        internal_1.TIMPLA_PROCESS.isDevelopment && wbConfigDev,
    ]
        .filter(Boolean)
        .map(function (cb) { return cb(); });
    var webpackConfig = wbmerge.apply(void 0, __spread(configs, [{
            mode: mode,
        }]));
    // Allow customisation of webpack config via task config method
    var _a = jsConfig.customizeWebpackConfig, customizeWebpackConfig = _a === void 0 ? function (_a) {
        var w = _a.webpackConfig;
        return w;
    } : _a;
    var fullWebpackConfig = customizeWebpackConfig({
        webpack: webpack,
        projectDestPath: internal_1.projectDestPath,
        projectSrcPath: internal_1.projectSrcPath,
        timplaConfig: timplaConfig,
        timplaProcess: internal_1.TIMPLA_PROCESS,
        webpackConfig: webpackConfig,
        webpackMerge: wbmerge,
    });
    internal_1.devLoggerTitle('Timpla process');
    internal_1.devLogger('%O', internal_1.TIMPLA_PROCESS, os.EOL);
    internal_1.devLoggerTitle('Webpack config');
    internal_1.devLogger('%O', fullWebpackConfig, os.EOL);
    if (internal_1.TIMPLA_PROCESS.isDevelopment) {
        logHmrMessage(useDevHmr);
    }
    // If MEASURE=any, wrap the whole config with SpeedMeasure
    // This allows us to visualiase how loaders and modules affect
    // Webpack builds
    // @TODO - buggy?
    if (internal_1.TIMPLA_PROCESS.MEASURE) {
        var SpeedMeasurePlugin = internal_1.lazyImport('speed-measure-webpack-plugin');
        var smp = new SpeedMeasurePlugin();
        return smp.wrap(fullWebpackConfig);
    }
    return fullWebpackConfig;
}; };
//# sourceMappingURL=webpackMultiConfig.js.map